@@ -1,101 +1,27 @@
 package Net::Google::Spreadsheets;
 use Moose;
+use namespace::clean -except => 'meta';
 use 5.008001;
 
-extends 'Net::Google::Spreadsheets::Base';
+with 
+    'Net::Google::Spreadsheets::Role::Base',
+    'Net::Google::Spreadsheets::Role::Service';
 
-use Carp;
-use Net::Google::AuthSub;
-use Net::Google::Spreadsheets::UserAgent;
-use Net::Google::Spreadsheets::Spreadsheet;
+our $VERSION = '0.06';
 
-our $VERSION = '0.05';
-
-BEGIN {
-    $XML::Atom::DefaultVersion = 1;
-}
-
-has +contents => (
+has spreadsheet_feed => (
+    traits => ['Net::Google::Spreadsheets::Traits::Feed'],
     is => 'ro',
-    default => 'http://spreadsheets.google.com/feeds/spreadsheets/private/full'
-);
-
-has +service => (
-    is => 'ro',
-    default => sub {return $_[0]},
-);
-
-has source => (
     isa => 'Str',
-    is => 'ro',
-    required => 1,
-    default => sub { __PACKAGE__.'-'.$VERSION },
+    default => 'http://spreadsheets.google.com/feeds/spreadsheets/private/full',
+    entry_class => 'Net::Google::Spreadsheets::Spreadsheet',
 );
 
-has username => ( isa => 'Str', is => 'ro', required => 1 );
-has password => ( isa => 'Str', is => 'ro', required => 1 );
+__PACKAGE__->meta->make_immutable;
 
-has ua => (
-    isa => 'Net::Google::Spreadsheets::UserAgent',
-    is => 'ro',
-    handles => [qw(request feed entry post put)],
-);
+sub _build_service {return $_[0]}
 
-sub BUILD {
-    my $self = shift;
-    my $authsub = Net::Google::AuthSub->new(
-        service => 'wise',
-        source => $self->source,
-    );
-    my $res = $authsub->login(
-        $self->username,
-        $self->password,
-    );
-    unless ($res && $res->is_success) {
-        croak 'Net::Google::AuthSub login failed';
-    } 
-    $self->{ua} = Net::Google::Spreadsheets::UserAgent->new(
-        source => $self->source,
-        auth => $res->auth,
-    );
-}
-
-sub spreadsheets {
-    my ($self, $args) = @_;
-    if ($args->{key}) {
-        my $atom = eval {$self->entry($self->contents.'/'.$args->{key})};
-        $atom or return;
-        return Net::Google::Spreadsheets::Spreadsheet->new(
-            atom => $atom,
-            service => $self,
-        );
-    } else {
-        my $cond = $args->{title} ?
-        {
-            title => $args->{title},
-            'title-exact' => 'true'
-        } : {};
-        my $feed = $self->feed(
-            $self->contents,
-            $cond,
-        );
-        return grep {
-            (!%$args && 1)
-            ||
-            ($args->{title} && $_->title eq $args->{title})
-        } map {
-            Net::Google::Spreadsheets::Spreadsheet->new(
-                atom => $_,
-                service => $self
-            )
-        } $feed->entries;
-    }
-}
-
-sub spreadsheet {
-    my ($self, $args) = @_;
-    return ($self->spreadsheets($args))[0];
-}
+sub _build_source { return __PACKAGE__. '-' . $VERSION }
 
 1;
 __END__
@@ -187,6 +113,43 @@ Net::Google::Spreadsheets - A Perl module for using Google Spreadsheets API.
     }
   );
 
+  # delete the row
+  $row->delete;
+
+  # delete the worksheet
+  $worksheet->delete;
+
+  # create a table
+  my $table = $spreadsheet->add_table(
+    {
+        worksheet => $new_worksheet,
+        columns => ['name', 'nick', 'mail address', 'age'],
+    }
+  );
+
+  # add a record
+  my $record = $table->add_record(
+    {
+        name => 'Nobuo Danjou',
+        nick => 'lopnor',
+        'mail address' => 'nobuo.danjou@gmail.com',
+        age  => '33',
+    }
+  );
+
+  # find a record
+  my $found = $table->record(
+    {
+        sq => '"mail address" = "nobuo.danjou@gmail.com"'
+    }
+  );
+
+  # delete it
+  $found->delete;
+
+  # delete table
+  $table->delete;
+
 =head1 DESCRIPTION
 
 Net::Google::Spreadsheets is a Perl module for using Google Spreadsheets API.
@@ -238,15 +201,53 @@ http://spreadsheets.google.com/ccc?key=key
 
 Returns first item of spreadsheets(\%condition) if available.
 
+=head1 TESTING
+
+To test this module, you have to prepare as below.
+
+=over 4
+
+=item create a spreadsheet by hand
+
+Go to L<http://docs.google.com> and create a spreadsheet.
+
+=item set SPREADSHEET_TITLE environment variable
+
+  export SPREADSHEET_TITLE='my test spreadsheet'
+
+or so.
+
+=item set username and password for google.com via Config::Pit
+
+install Config::Pit and type 
+
+  ppit set google.com
+
+then some editor comes up and type your username and password like
+
+  ---
+  username: myname@gmail.com
+  password: foobarbaz
+
+=item run tests
+
+as always,
+
+  perl Makefile.PL
+  make
+  make test
+
+=back
+
 =head1 AUTHOR
 
 Nobuo Danjou E<lt>nobuo.danjou@gmail.comE<gt>
 
 =head1 SEE ALSO
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/developers_guide_protocol.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/developers_guide_protocol.html>
 
-L<http://code.google.com/intl/en/apis/spreadsheets/docs/2.0/reference.html>
+L<http://code.google.com/intl/en/apis/spreadsheets/docs/3.0/reference.html>
 
 L<Net::Google::AuthSub>
 
@@ -258,6 +259,10 @@ L<Net::Google::Spreadsheets::Cell>
 
 L<Net::Google::Spreadsheets::Row>
 
+L<Net::Google::Spreadsheets::Table>
+
+L<Net::Google::Spreadsheets::Record>
+
 =head1 LICENSE
 
 This library is free software; you can redistribute it and/or modify
